@@ -25,9 +25,11 @@
 #include "bitmap.h"
 
 
+namespace {
+
 char pbm_getrawbyte( FILE * f ) throw( Bitmap::Error )
   {
-  int ch = getc( f );
+  int ch = std::getc( f );
 
   if( ch == EOF )
     throw Bitmap::Error( "end-of-file reading pbm file.\n" );
@@ -35,6 +37,7 @@ char pbm_getrawbyte( FILE * f ) throw( Bitmap::Error )
   return static_cast< char > (ch);
   }
 
+
 char pbm_getc( FILE * f ) throw()
   {
   char ch;
@@ -42,39 +45,43 @@ char pbm_getc( FILE * f ) throw()
 
   do {
     ch = pbm_getrawbyte( f );
-    if( ch == '\n' ) comment = false;
-    else if( ch == '#' ) comment = true;
+    if( ch == '#' ) comment = true;
+    else if( ch == '\n' ) comment = false;
     }
   while( comment );
   return ch;
   }
 
+
 int pbm_getint( FILE * f ) throw( Bitmap::Error )
   {
   char ch;
   int i = 0;
 
-  do ch = pbm_getc( f ); while( isspace( ch ) );
-  if( !isdigit( ch ) )
+  do ch = pbm_getc( f ); while( std::isspace( ch ) );
+  if( !std::isdigit( ch ) )
     throw Bitmap::Error( "junk in pbm file where an integer should be.\n" );
   do { i = (i * 10) + (ch - '0'); ch = pbm_getc( f ); }
-  while( isdigit( ch ) );
+  while( std::isdigit( ch ) );
   return i;
   }
 
+
 bool pbm_getbit( FILE * f ) throw( Bitmap::Error )
   {
   char ch;
 
-  do ch = pbm_getc( f ); while( isspace( ch ) );
+  do ch = pbm_getc( f ); while( std::isspace( ch ) );
 
   if( ch == '0' ) return false;
   if( ch == '1' ) return true;
   throw Bitmap::Error( "junk in pbm file where bits should be.\n" );
   }
 
+} // end namespace
 
-Bitmap::Bitmap( FILE * f ) throw( Error )
+
+Bitmap::Bitmap( FILE * f, bool invert ) throw( Error )
   {
   char filetype = 0;
 
@@ -90,24 +97,40 @@ Bitmap::Bitmap( FILE * f ) throw( Error )
   _height = pbm_getint( f );
 
   data.resize( _height );
-  for( int row = 0; row < _height; ++row )
-    {
-    data[row].resize( _width );
 
-    if( filetype == '1' )
+  if( filetype == '1' )
+    {
+    for( int row = 0; row < _height; ++row )
       {
-      for( int col = 0; col < _width; ++col )
-        data[row][col] = pbm_getbit( f );
+      data[row].resize( _width );
+      if( invert )
+        for( int col = 0; col < _width; ++col )
+          data[row][col] = !pbm_getbit( f );
+      else
+        for( int col = 0; col < _width; ++col )
+          data[row][col] = pbm_getbit( f );
       }
-    else
+    }
+  else
+    {
+    for( int row = 0; row < _height; ++row )
       {
+      data[row].resize( _width );
       unsigned char byte = 0, mask = 0;
-      for( int col = 0; col < _width; ++col )
-        {
-        if( mask == 0 ) { mask = 0x80; byte = pbm_getrawbyte( f ); }
-        data[row][col] = ( byte & mask ) ? true : false;
-        mask >>= 1;
-        }
+      if( invert )
+        for( int col = 0; col < _width; ++col )
+          {
+          if( mask == 0 ) { mask = 0x80; byte = pbm_getrawbyte( f ); }
+          data[row][col] = !( byte & mask );
+          mask >>= 1;
+          }
+      else
+        for( int col = 0; col < _width; ++col )
+          {
+          if( mask == 0 ) { mask = 0x80; byte = pbm_getrawbyte( f ); }
+          data[row][col] = ( byte & mask );
+          mask >>= 1;
+          }
       }
     }
   }
@@ -116,7 +139,7 @@ Bitmap::Bitmap( FILE * f ) throw( Error )
 Bitmap::Bitmap( const Bitmap & source, const Rectangle & r ) throw()
   {
   if( r.right() >= source._width || r.bottom() >= source._height )
-    internal_error( "bad parameter building a Bitmap from another one" );
+    Ocrad::internal_error( "bad parameter building a Bitmap from another one" );
 
   _width = r.right() - r.left() + 1;
   _height = r.bottom() - r.top() + 1;
@@ -136,7 +159,7 @@ Bitmap::Bitmap( const Bitmap & source, int th, int scale ) throw()
   {
   if( scale < 2 || scale > source._width || scale > source._height ||
       th >= scale * scale )
-    internal_error( "bad parameter building a reduced Bitmap" );
+    Ocrad::internal_error( "bad parameter building a reduced Bitmap" );
 
   if( th < 0 )
     {
@@ -145,7 +168,7 @@ Bitmap::Bitmap( const Bitmap & source, int th, int scale ) throw()
       for( int col = 0; col < source._width; ++col )
         if( source.data[row][col] ) ++dots;
     th = (scale * scale * dots) / (source._width * source._height);
-    fprintf( stderr, "automatic threshold = %d\n", th );
+    std::fprintf( stderr, "automatic threshold = %d\n", th );
     }
 
   _width = source._width / scale;
@@ -167,7 +190,7 @@ Bitmap::Bitmap( const Bitmap & source, int th, int scale ) throw()
             if( (col * scale) + j < source._width )
               if( source.data[(row * scale) + i][(col * scale) + j] )
                 ++counter;
-      data[row][col] = ( counter > th ? true : false );
+      data[row][col] = ( counter > th );
       }
     }
   }
@@ -227,31 +250,31 @@ Bitmap::Bitmap( const Bitmap & source, type t ) throw( Error )
     if( row == _height ) throw Error( "image is not clear enough.\n" );
     floodfill4( source, *this, row, _width/2 );
     }
-  else internal_error( "type not implemented building a 'special' Bitmap" );
+  else Ocrad::internal_error( "type not implemented building a 'special' Bitmap" );
   }
 
 
 int Bitmap::analyse_layout( int layout_level ) throw()
   {
   if( _rectangle_vector.size() != 0 )
-    internal_error( "analyse_layout called twice" );
+    Ocrad::internal_error( "analyse_layout called twice" );
 
   Rectangle r( 0, 0, _width - 1, _height - 1 );
   _rectangle_vector.push_back( r );
   if( layout_level <= 0 ) return _rectangle_vector.size();
 
-  find_columns();
+  find_columns( 0 );
   if( layout_level <= 1 ) return _rectangle_vector.size();
 
-  while( find_rows() ) find_columns();
+  find_rows_and_columns();
   return _rectangle_vector.size();
   }
 
 
-bool Bitmap::find_columns() throw()
+bool Bitmap::find_columns( int rindex ) throw()
   {
   bool split = false;
-  for( int rindex = 0; rindex < rectangles(); )
+//  for( int rindex = 0; rindex < rectangles(); )
     {
     int columns = 0;
     const Rectangle & r = _rectangle_vector[rindex];
@@ -274,8 +297,15 @@ bool Bitmap::find_columns() throw()
     int left, right, col_width, gap_width;
     for( left = 0; left < r.width(); ++left )	// cut left border
       if( h_outline[left] >= threshold_gap ) break;
+    if( left < _width / 10 )			// cut left border noise
+      {
+      int left2 = left + gapmin;
+      for( ; left2 < r.width(); ++left2 )
+        if( h_outline[left2] >= threshold_gap ) break;
+      if( left2 > left + ( 2 * gapmin ) ) left = left2;
+      }
     right = left + 1;
-    left = ( left >= gapmin ) ? left / 2 : 0;
+    left = ( left >= gapmin ) ? left - gapmin : 0;
 
     while( true )
       {
@@ -314,15 +344,15 @@ bool Bitmap::find_columns() throw()
   }
 
 
-bool Bitmap::find_rows() throw()
+bool Bitmap::find_rows_and_columns() throw()
   {
   bool split = false;
-  for( int rindex = 0; rindex < rectangles(); ++rindex )
+  for( int rindex = 0; rindex < rectangles(); )
     {
     Rectangle & r = _rectangle_vector[rindex];
     const int rowmin = ( _height > 1000 ) ? _height / 10 : 100;
     const int gapmin = ( _height > 2000 ) ? _height / 100 : 20;
-    if( r.height() < ( 2 * rowmin ) + gapmin ) continue;
+    if( r.height() < ( 2 * rowmin ) + gapmin ) { ++rindex; continue; }
     std::vector< int > v_outline;
     v_outline.resize( r.height() );
 
@@ -342,7 +372,7 @@ bool Bitmap::find_rows() throw()
       if( v_outline[top] > threshold_gap ) break;
     for( bottom = r.height() - 1; bottom > top; --bottom ) // cut bottom border
       if( v_outline[bottom] > threshold_gap ) break;
-    if( bottom - top < 2 * rowmin ) continue;
+    if( bottom - top < 2 * rowmin ) { ++rindex; continue; }
 
     int t, b, best_top = 0, best_bottom = 0, gaps = 0;
     for( t = top + gapmin; t < bottom; ++t )  // look for top of gap
@@ -355,19 +385,19 @@ bool Bitmap::find_rows() throw()
           { best_top = t; best_bottom = b; }
         t = b;
         }
-    if( gaps == 0 )
-      {
+    if( gaps == 0 )	// eliminates images
       _rectangle_vector.erase( _rectangle_vector.begin() + rindex );
-      --rindex; continue;
-      }
-    if( best_bottom - best_top >= gapmin )
+    else if( best_bottom - best_top >= gapmin )
       {
       int mid = r.top() + ( ( best_top + best_bottom ) / 2 );
       Rectangle r1( r.left(), mid + 1, r.right(), r.bottom() );
-      r.bottom( mid ); ++rindex;
-      _rectangle_vector.insert( _rectangle_vector.begin() + rindex, r1 );
+      r.bottom( mid );
+      _rectangle_vector.insert( _rectangle_vector.begin() + rindex + 1, r1 );
+      find_columns( rindex + 1 );
+      find_columns( rindex );
       split = true;
       }
+    else ++rindex;
     }
   return split;
   }
@@ -397,8 +427,8 @@ int Bitmap::horizontalify( bool verbose ) throw()
 
 if( verbose )
   {
-  fprintf( stderr, "mean = %ld\n", mean );
-  fprintf( stderr, "slope   0   variance %Ld\n", variance_best );
+  std::fprintf( stderr, "mean = %ld\n", mean );
+  std::fprintf( stderr, "slope   0   variance %Ld\n", variance_best );
   }
 
   bool try_inc = true, try_dec = true;
@@ -424,7 +454,7 @@ if( verbose )
       else if( variance < variance_best - (variance_best / 10) )
         try_inc = false;
       if( verbose )
-        fprintf( stderr, "slope %3d   variance %Ld\n", slope, variance );
+        std::fprintf( stderr, "slope %3d   variance %Ld\n", slope, variance );
       }
 
     if( try_dec )
@@ -445,7 +475,7 @@ if( verbose )
       else if( variance < variance_best - (variance_best / 10) )
         try_dec = false;
       if( verbose )
-        fprintf( stderr, "slope %3d   variance %Ld\n", -slope, variance );
+        std::fprintf( stderr, "slope %3d   variance %Ld\n", -slope, variance );
       }
     }
 
@@ -488,15 +518,15 @@ if( verbose )
 void Bitmap::save( FILE * f, char filetype ) const throw()
   {
   if( filetype != '1' && filetype != '4' ) filetype = '4';
-  fprintf( f, "P%c\n%d %d\n", filetype, _width, _height );
+  std::fprintf( f, "P%c\n%d %d\n", filetype, _width, _height );
 
   for( int row = 0; row < _height; ++row )
     {
     if( filetype == '1' )
       {
       for( int col = 0; col < _width; ++col )
-        putc( data[row][col] ? '1' : '0', f );
-      putc( '\n', f );
+        std::putc( data[row][col] ? '1' : '0', f );
+      std::putc( '\n', f );
       }
     else
       {
@@ -505,10 +535,10 @@ void Bitmap::save( FILE * f, char filetype ) const throw()
         {
         if( data[row][col] ) byte |= mask;
         mask >>= 1;
-        if( mask == 0 ) { putc( byte, f ); byte = 0; mask = 0x80; }
+        if( mask == 0 ) { std::putc( byte, f ); byte = 0; mask = 0x80; }
         }
-      if( mask != 0x80 ) putc( byte, f );  // incomplete byte at end of row
+      if( mask != 0x80 ) std::putc( byte, f );  // incomplete byte at end of row
       }
     }
-  fclose( f );
+  std::fclose( f );
   }
