@@ -25,8 +25,8 @@ class Bitmap
   std::vector< std::vector< bool > > data;
   std::vector< Rectangle > _rectangle_vector;
 
-  bool find_columns() throw();
-  bool find_rows() throw();
+  bool find_columns( int rindex ) throw();
+  bool find_rows_and_columns() throw();
 
 public:
   struct Error
@@ -37,7 +37,8 @@ public:
 
   enum type { vertical_histogram, horizontal_histogram, connected_ground };
 
-  Bitmap( FILE * ) throw( Error );	// Creates a Bitmap from a pbm file
+  // Creates a Bitmap from a pbm file
+  Bitmap( FILE * f, bool invert ) throw( Error );
   // Creates a Bitmap from another Bitmap
   Bitmap( const Bitmap & source, const Rectangle & r ) throw();
   // Creates a reduced bitmap
