@@ -26,58 +26,11 @@
 #include "blockmap.h"
 
 
-Block * Blockmap::create_block( const Rectangle & r, int id ) throw()
-  {
-  _block_list.push_back( Block( r, *this, id ) );
-  return &_block_list.back();
-  }
+namespace {
 
+int generate_black_id() { static int next = 0; return ++next; }
 
-void Blockmap::delete_block( int id ) throw()
-  {
-  std::list< Block >::reverse_iterator p = _block_list.rbegin();
-  for( ; p != _block_list.rend(); ++p )
-    if( id == p->id() )
-      {
-      for( int row = p->top(); row <= p->bottom(); ++row )
-        for( int col = p->left(); col <= p->right(); ++col )
-          if( data[row][col] == id ) data[row][col] = 0;
-      _block_list.erase( --(p.base()) ); return;
-      }
-
-  internal_error( "delete_block, lost block" );
-  }
-
-
-Block * Blockmap::join_blocks( int id1, int id2 ) throw()
-  {
-  if( abs( id1 ) > abs( id2 ) )
-    { int temp = id1; id1 = id2; id2 = temp; }
-
-  std::list< Block >::reverse_iterator p1 = _block_list.rbegin();
-  std::list< Block >::reverse_iterator p2 = _block_list.rbegin();
-  while( p1 != _block_list.rend() && id1 != p1->id() ) ++p1;
-  while( p2 != _block_list.rend() && id2 != p2->id() ) ++p2;
-  if( p1 == _block_list.rend() || p2 == _block_list.rend() )
-    internal_error( "join_blocks, lost block" );
-
-  for( int row = p2->top(); row <= p2->bottom(); ++row )
-    for( int col = p2->left(); col <= p2->right(); ++col )
-      if( data[row][col] == id2 ) data[row][col] = id1;
-  p1->add_rectangle( *p2 );
-  _block_list.erase( --(p2.base()) );
-  return &(*p1);
-  }
-
-
-Block * Blockmap::add_point_to_block( int row, int col, int id ) throw()
-  {
-  std::list< Block >::reverse_iterator p = _block_list.rbegin();
-  for( ; p != _block_list.rend(); ++p )
-    if( id == p->id() ) { p->add_point( row, col ); return &(*p); }
-
-  internal_error( "add_point_to_block, lost block" );
-  }
+int generate_white_id() { static int next = 0; return --next; }
 
 
 void delete_small_blocks( std::list< Block > & block_list ) throw()
@@ -88,7 +41,7 @@ void delete_small_blocks( std::list< Block > & block_list ) throw()
     std::list< Block >::iterator next = p;
     ++next;
     if( p->height() <= 4 && p->width() <= 4 &&
-        ( ( p->height() <= 2 && p->width() <= 2 ) || p->area() <= 4 ) )
+        ( ( p->height() <= 2 && p->width() <= 2 ) || p->area() <= 5 ) )
       block_list.erase( p );
     p = next;
     }
@@ -121,35 +74,6 @@ void delete_wide_blocks( std::list< Block > & block_list, int width ) throw()
   }
 
 
-void hierarchize_blocks( std::list< Block > & block_list ) throw()
-  {
-  std::list< Block >::iterator p1 = block_list.begin();
-  for( ; p1 != block_list.end(); ++p1 ) if( p1->id() > 0 )
-    {
-    std::list< Block >::iterator p2 = p1; ++p2;
-    while( p2 != block_list.end() )
-      {
-      if( p2->top() >= p1->bottom() ) break;
-      std::list< Block >::iterator next = p2;
-      ++next;
-      if( p1->includes( *p2 ) )
-        {
-        if( p2->id() < 0 )
-          p1->_block_list.splice( p1->_block_list.end(), block_list, p2 );
-        else
-          {
-          std::list< Block >::iterator p3 = p1->_block_list.begin();
-          for( ; p3 != p1->_block_list.end(); ++p3 )
-            if( p3->includes( *p2 ) )
-              p3->_block_list.splice( p3->_block_list.end(), block_list, p2 );
-          }
-        }
-      p2 = next;
-      }
-    }
-  }
-
-
 void order_blocks( std::list< Block > & block_list ) throw()
   {
   std::list< Block >::iterator p1 = block_list.begin();
@@ -185,17 +109,98 @@ void remove_top_bottom_noise( std::list< Block > & block_list ) throw()
     }
   }
 
+} // end namespace
 
-int generate_black_id() { static int next = 0; return ++next; }
 
-int generate_white_id() { static int next = 0; return --next; }
+void hierarchize_blocks( std::list< Block > & block_list ) throw()
+  {
+  std::list< Block >::iterator p1 = block_list.begin();
+  for( ; p1 != block_list.end(); ++p1 ) //if( p1->id() > 0 )
+    {
+    std::list< Block >::iterator p2 = p1; ++p2;
+    while( p2 != block_list.end() )
+      {
+      if( p2->top() >= p1->bottom() ) break;
+      std::list< Block >::iterator next = p2;
+      ++next;
+      if( p1->includes( *p2 ) )
+        {
+        if( p1->id() * p2->id() < 0 )
+          p1->_block_list.splice( p1->_block_list.end(), block_list, p2 );
+        else
+          {
+          std::list< Block >::iterator p3 = p1->_block_list.begin();
+          for( ; p3 != p1->_block_list.end(); ++p3 )
+            if( p3->includes( *p2 ) )
+              p3->_block_list.splice( p3->_block_list.end(), block_list, p2 );
+          }
+        }
+      p2 = next;
+      }
+    }
+  }
+
+
+Block * Blockmap::create_block( const Rectangle & r, int id ) throw()
+  {
+  _block_list.push_back( Block( r, *this, id ) );
+  return &_block_list.back();
+  }
+
+
+void Blockmap::delete_block( int id ) throw()
+  {
+  std::list< Block >::reverse_iterator p = _block_list.rbegin();
+  for( ; p != _block_list.rend(); ++p )
+    if( id == p->id() )
+      {
+      for( int row = p->top(); row <= p->bottom(); ++row )
+        for( int col = p->left(); col <= p->right(); ++col )
+          if( data[row][col] == id ) data[row][col] = 0;
+      _block_list.erase( --(p.base()) ); return;
+      }
+
+  Ocrad::internal_error( "delete_block, lost block" );
+  }
+
+
+Block * Blockmap::join_blocks( int id1, int id2 ) throw()
+  {
+  if( std::abs( id1 ) > std::abs( id2 ) )
+    { int temp = id1; id1 = id2; id2 = temp; }
+
+  std::list< Block >::reverse_iterator p1 = _block_list.rbegin();
+  std::list< Block >::reverse_iterator p2 = _block_list.rbegin();
+  while( p1 != _block_list.rend() && id1 != p1->id() ) ++p1;
+  while( p2 != _block_list.rend() && id2 != p2->id() ) ++p2;
+  if( p1 == _block_list.rend() || p2 == _block_list.rend() )
+    Ocrad::internal_error( "join_blocks, lost block" );
+
+  for( int row = p2->top(); row <= p2->bottom(); ++row )
+    for( int col = p2->left(); col <= p2->right(); ++col )
+      if( data[row][col] == id2 ) data[row][col] = id1;
+  p1->add_rectangle( *p2 );
+  _block_list.erase( --(p2.base()) );
+  return &(*p1);
+  }
+
+
+Block * Blockmap::add_point_to_block( int row, int col, int id ) throw()
+  {
+  std::list< Block >::reverse_iterator p = _block_list.rbegin();
+  for( ; p != _block_list.rend(); ++p )
+    if( id == p->id() ) { p->add_point( row, col ); return &(*p); }
+
+  Ocrad::internal_error( "add_point_to_block, lost block" );
+  }
+
 
 Blockmap::Blockmap( const Bitmap & page_image, int rindex, int debug_level )
                                                                      throw()
   {
   if( rindex < 0 ||
       rindex >= static_cast<int>(page_image.rectangle_vector().size()) )
-    internal_error( "Blockmap, text rectangle index out of range" );
+    Ocrad::internal_error( "Blockmap, text rectangle index out of range" );
 
   const Rectangle & r = page_image.rectangle_vector()[rindex];
   int row0 = r.top();
@@ -280,8 +285,8 @@ Blockmap::Blockmap( const Bitmap & page_image, int rindex, int debug_level )
 
 void Blockmap::print( FILE * outfile, int debug_level ) const throw()
   {
-  fprintf( outfile, "%d blocks\n", _block_list.size() );
-  fprintf( outfile, "%d %d\n\n", _width, _height );
+  std::fprintf( outfile, "%d blocks\n", _block_list.size() );
+  std::fprintf( outfile, "%d %d\n\n", _width, _height );
 
   int sp = (debug_level & 1) ? 0 : -1;
   std::list< Block >::const_iterator p = _block_list.begin();
