@@ -18,12 +18,12 @@
 
 #include <cstdio>
 #include <list>
-#include <map>
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
 #include "block.h"
 #include "character.h"
+#include "iso_8859_1.h"
 
 
 // Return the area of the main blocks only (no recursive)
@@ -39,7 +39,7 @@ int Character::area() const throw()
 void Character::add_block( const Block & b ) throw()
   {
   if( _blockmap != b.blockmap() )
-    internal_error( "can't add block to character of different blockmaps.\n" );
+    Ocrad::internal_error( "can't add block to character of different blockmaps.\n" );
   add_rectangle( b );
   if( _block_list.size() == 0 ) { _block_list.push_back( b ); return; }
   std::list< Block >::iterator p = _block_list.begin();
@@ -58,7 +58,7 @@ void Character::add_guess( unsigned char ch, int value ) throw()
 void Character::insert_guess( int i, unsigned char ch, int value ) throw()
   {
   if( i < 0 || i > guesses() )
-    internal_error( "insert_guess, index out of bounds" );
+    Ocrad::internal_error( "insert_guess, index out of bounds" );
   _guess.insert( _guess.begin() + i, Guess( ch, value ) );
   }
 
@@ -66,7 +66,7 @@ void Character::insert_guess( int i, unsigned char ch, int value ) throw()
 void Character::delete_guess( int i ) throw()
   {
   if( i < 0 || i >= guesses() )
-    internal_error( "delete_guess, index out of bounds" );
+    Ocrad::internal_error( "delete_guess, index out of bounds" );
   _guess.erase( _guess.begin() + i );
   }
 
@@ -78,23 +78,33 @@ void Character::only_guess( unsigned char ch, int value ) throw()
 void Character::swap_guesses( int i, int j ) throw()
   {
   if( i < 0 || i >= guesses() || j < 0 || j >= guesses() )
-    internal_error( "swap_guesses, index out of bounds" );
-  Guess tmp = _guess[i]; _guess[i] = _guess[j]; _guess[j] = tmp;
+    Ocrad::internal_error( "swap_guesses, index out of bounds" );
+//  Guess tmp = _guess[i]; _guess[i] = _guess[j]; _guess[j] = tmp;
+  unsigned char ch = _guess[i].ch;
+  _guess[i].ch = _guess[j].ch; _guess[j].ch = ch;
   }
 
 
 const Character::Guess & Character::guess( int i ) const throw()
   {
   if( i < 0 || i >= guesses() )
-    internal_error( "guess, index out of bounds" );
+    Ocrad::internal_error( "guess, index out of bounds" );
   return _guess[i];
   }
 
 
+bool Character::isalnum() const throw()
+  {
+  for( int i = 0; i < guesses(); ++i )
+    if( ISO_8859_1::isalnum( _guess[i].ch ) ) return true;
+  return false;
+  }
+
+
 void Character::join( Character & c ) throw()
   {
   if( _blockmap != c._blockmap )
-    internal_error( "can't join characters of different blockmaps.\n" );
+    Ocrad::internal_error( "can't join characters of different blockmaps.\n" );
   std::list< Block >::const_iterator p = c._block_list.begin();
   for( ; p != c._block_list.end(); ++p ) add_block( *p );
   }
@@ -105,17 +115,18 @@ void Character::print( FILE * outfile, bool graph, bool recursive )
   {
   if( graph || recursive )
     {
-    fprintf( outfile, "%d guesses    ", guesses() );
+    std::fprintf( outfile, "%d guesses    ", guesses() );
     for( int i = 0; i < guesses(); ++i )
-      fprintf( outfile, "guess '%c', confidence %d    ", _guess[i].ch,
-                                                         _guess[i].value );
-    fprintf( outfile, "\n" );
+      std::fprintf( outfile, "guess '%c', confidence %d    ",
+                    _guess[i].ch, _guess[i].value );
+    std::fputs( "\n", outfile );
     }
   if( graph )
     {
     const Block * b = ( _block_list.size() ) ? &_block_list.front() : 0;
-    fprintf( outfile, "height = %d, width = %d, vcenter = %d, hcenter = %d, black area = %d%%\n",
-		height(), width(), vcenter(), hcenter(), (area() * 100) / (height() * width()) );
+    std::fprintf( outfile, "height = %d, width = %d, vcenter = %d, hcenter = %d, black area = %d%%\n",
+                  height(), width(), vcenter(), hcenter(),
+                  ( area() * 100 ) / ( height() * width() ) );
     for( int row = top(); row <= bottom(); ++row)
       {
       for( int col = left(); col <= right(); ++col)
@@ -128,33 +139,33 @@ void Character::print( FILE * outfile, bool graph, bool recursive )
             if( p->compare_id( b->id( row, col ), recursive ) )
               { ch = (ch == '-') ? '+' : 'O'; break; }
           }
-        fprintf( outfile, " %c", ch );
+        std::fprintf( outfile, " %c", ch );
         }
-      fprintf( outfile, "\n" );
+      std::fputs( "\n", outfile );
       }
-    fprintf( outfile, "\n" );
+    std::fputs( "\n", outfile );
     }
   else
     {
     if( recursive )
       {
       for( int i = 0; i < guesses(); ++i )
-        fprintf( outfile, "%c %4d,    ", _guess[i].ch, _guess[i].value );
-      fputs( "\n", outfile );
+        std::fprintf( outfile, "%c %4d,    ", _guess[i].ch, _guess[i].value );
+      std::fputs( "\n", outfile );
       }
-    else if( guesses() ) fputc( _guess[0].ch, outfile );
-    else fputc( '_', outfile );
+    else if( guesses() ) std::putc( _guess[0].ch, outfile );
+    else std::putc( '_', outfile );
     }
   }
 
 
 void Character::xprint( FILE * exportfile ) const throw()
   {
-  fprintf( exportfile, "%d %d %d %d;%d",
-           left(), top(), width(), height(), guesses() );
+  std::fprintf( exportfile, "%d %d %d %d;%d",
+                left(), top(), width(), height(), guesses() );
 
   for( int i = 0; i < guesses(); ++i )
-    fprintf( exportfile, ",'%c'%d", _guess[i].ch, _guess[i].value );
+    std::fprintf( exportfile, ",'%c'%d", _guess[i].ch, _guess[i].value );
 
-  fprintf( exportfile, "\n" );
+  std::fputs( "\n", exportfile );
   }
