@@ -16,22 +16,23 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <algorithm>
 #include <cstdio>
 #include <cstdlib>
 #include "common.h"
 
 
-void internal_error( const char * s ) throw()
+void Ocrad::internal_error( const char * s ) throw()
   {
-  fprintf( stderr, "%s internal error: %s.\n", PROGRAM_NAME, s );
+  std::fprintf( stderr, "%s internal error: %s.\n", PROGRAM_NAME, s );
   exit( 3 );
   }
 
 
-bool similar( int a, int b, int percent_dif ) throw()
+bool Ocrad::similar( int a, int b, int percent_dif ) throw()
   {
-  int difference = abs( a - b );
+  int difference = std::abs( a - b );
   if( percent_dif > 0 && difference <= 1 ) return true;
-  int max_abs = ( abs(a) >= abs(b) ? abs(a) : abs(b) );
+  int max_abs = std::max( std::abs(a), std::abs(b) );
   return ( difference * 100 <= max_abs * percent_dif );
   }
