@@ -16,13 +16,15 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-// Date of this version: 2003-07-19
+// Date of this version: 2003-09-03
 
-#define _GNU_SOURCE 1
 #define PROGRAM_NAME    "ocrad"
-#define PROGRAM_VERSION "0.3"
+#define PROGRAM_VERSION "0.4"
 #define PROGRAM_YEAR    "2003"
 
 
-void internal_error( const char * s ) throw() __attribute__ ((noreturn));
-bool similar( int a, int b, int percent_dif ) throw();
+namespace Ocrad
+  {
+  void internal_error( const char * s ) throw() __attribute__ ((noreturn));
+  bool similar( int a, int b, int percent_dif ) throw();
+  }
