@@ -126,6 +126,12 @@ unsigned char ISO_8859_1::compose( unsigned char base_letter,
   }
 
 
+bool ISO_8859_1::isalnum( unsigned char ch ) throw()
+  {
+  return ( ISO_8859_1::isalpha( ch ) || std::isdigit( ch ) );
+  }
+
+
 bool ISO_8859_1::isalpha( unsigned char ch ) throw()
   {
   return ( std::isalpha( ch ) || base_letter( ch ) );
