@@ -88,6 +88,7 @@ public:
               SOCIRCU = 244,	// latin small letter o with circumflex
               SOTILDE = 245,	// latin small letter o with tilde
               SODIAER = 246,	// latin small letter o with diaeresis
+              DIV     = 247,	// division sign
               SUGRAVE = 249,	// latin small letter u with grave
               SUACUTE = 250,	// latin small letter u with acute
               SUCIRCU = 251,	// latin small letter u with circumflex
@@ -99,6 +100,7 @@ public:
   static unsigned char base_letter( unsigned char ch ) throw();
   static unsigned char compose( unsigned char base_letter,
                                 unsigned char accent ) throw();
+  static bool isalnum( unsigned char ch ) throw();
   static bool isalpha( unsigned char ch ) throw();
   static bool isupper( unsigned char ch ) throw();
   static bool isvowel( unsigned char ch ) throw();
