@@ -39,44 +39,47 @@
 #include "textblock.h"
 
 
+namespace {
+
 void show_version()
   {
-  printf( "GNU %s version %s\n", PROGRAM_NAME, PROGRAM_VERSION );
-  printf( "Copyright (C) %s Antonio Diaz.\n", PROGRAM_YEAR );
-  printf( "This program is free software; you may redistribute it under the terms of\n" );
-  printf( "the GNU General Public License.  This program has absolutely no warranty.\n" );
+  std::printf( "GNU %s version %s\n", PROGRAM_NAME, PROGRAM_VERSION );
+  std::printf( "Copyright (C) %s Antonio Diaz.\n", PROGRAM_YEAR );
+  std::printf( "This program is free software; you may redistribute it under the terms of\n" );
+  std::printf( "the GNU General Public License.  This program has absolutely no warranty.\n" );
   }
 
 void show_error( char * msg )
   {
-  if( msg && msg[0] != 0 ) fprintf( stderr,"%s: %s\n", PROGRAM_NAME, msg );
-  fprintf( stderr,"Try `%s --help' for more information.\n", PROGRAM_NAME );
+  if( msg && msg[0] != 0 ) std::fprintf( stderr,"%s: %s\n", PROGRAM_NAME, msg );
+  std::fprintf( stderr,"Try `%s --help' for more information.\n", PROGRAM_NAME );
   }
 
 void show_help()
   {
-  printf( "GNU %s, Optical Character Recognition program\n", PROGRAM_NAME );
-  printf( "Reads pbm file(s), or standard input, and sends text to standard output\n" );
-  printf( "\nUsage: %s [options] [files]\n", PROGRAM_NAME );
-  printf( "Options:\n");
-  printf( "  -h, --help               display this help and exit\n");
-  printf( "  -V, --version            output version information and exit\n");
-  printf( "  -1, -4                   pbm output file type (debug)\n");
-  printf( "  -a, --append             append text to output file\n");
-  printf( "  -b, --block=<n>          process only the specified text block\n");
-  printf( "  -C, --copy               'copy' input to output (debug)\n");
-  printf( "  -D, --debug=<level>      (0-100) output intermediate data (debug)\n");
-  printf( "  -f, --force              force overwrite of output file\n");
-//  printf( "  -L, --level              level input image before doing OCR\n");
-  printf( "  -l, --layout=<n>         layout analysis, 0=none, 1=column, 2=full\n");
-  printf( "  -o <file>                place the output into <file>\n");
-//  printf( "  -R <scale>               reduce input image by <scale> (debug)\n");
-//  printf( "  -s <type>                make a 'special file' (debug)\n");
-//  printf( "  -t <threshold>           set threshold for -R option (debug)\n");
-  printf( "  -v, --verbose            be verbose\n");
-  printf( "  -x <file>                export OCR Results File to <file>\n");
+  std::printf( "GNU %s, Optical Character Recognition program\n", PROGRAM_NAME );
+  std::printf( "Reads pbm file(s), or standard input, and sends text to standard output\n" );
+  std::printf( "\nUsage: %s [options] [files]\n", PROGRAM_NAME );
+  std::printf( "Options:\n");
+  std::printf( "  -h, --help               display this help and exit\n");
+  std::printf( "  -V, --version            output version information and exit\n");
+  std::printf( "  -1, -4                   pbm output file type (debug)\n");
+  std::printf( "  -a, --append             append text to output file\n");
+  std::printf( "  -b, --block=<n>          process only the specified text block\n");
+  std::printf( "  -C, --copy               'copy' input to output (debug)\n");
+  std::printf( "  -D, --debug=<level>      (0-100) output intermediate data (debug)\n");
+  std::printf( "  -f, --force              force overwrite of output file\n");
+  std::printf( "  -i, --invert             invert image levels (white on black)\n");
+//  std::printf( "  -L, --level              level input image before doing OCR\n");
+  std::printf( "  -l, --layout=<n>         layout analysis, 0=none, 1=column, 2=full\n");
+  std::printf( "  -o <file>                place the output into <file>\n");
+//  std::printf( "  -R <scale>               reduce input image by <scale> (debug)\n");
+//  std::printf( "  -s <type>                make a 'special file' (debug)\n");
+//  std::printf( "  -t <threshold>           set threshold for -R option (debug)\n");
+  std::printf( "  -v, --verbose            be verbose\n");
+  std::printf( "  -x <file>                export OCR Results File to <file>\n");
 
-  printf( "\nReport bugs to bug-ocrad@gnu.org\n");
+  std::printf( "\nReport bugs to bug-ocrad@gnu.org\n");
   }
 
 
@@ -96,26 +99,29 @@ void ocr( const Blockmap & blockmap, FILE * outfile, FILE * exportfile,
 
   if( debug_level >= 86 )
     {
-    bool graph = debug_level >= 88 ? true : false;
-    bool recursive = (debug_level & 1) ? true : false;
+    bool graph = ( debug_level >= 88 );
+    bool recursive = ( debug_level & 1 );
     textblock.print( outfile, graph, recursive );
     return;
     }
 
+//  if( textblock.alnum_percent() > 50 )
   textblock.print( outfile, false, false );
   if( exportfile ) textblock.xprint( exportfile );
   }
 
+} // end namespace
+
 
 int main(int argc, char *argv[]) 
   {
   FILE *infile = stdin, *outfile = stdout, *exportfile = 0;
-  char *outfile_name = 0, *exportfile_name = 0;
+  char *infile_name = 0, *outfile_name = 0, *exportfile_name = 0;
   int rindex = -1, debug_level = 0, layout_level = 0;
   int scale = 0, threshold = -1;
   char filetype = 0, specialtype = 0;
-  bool append = false, copy = false, force = false, level = false;
-  bool verbose = false;
+  bool append = false, copy = false, force = false, invert = false;
+  bool level = false, verbose = false;
 
   // scan arguments
 
@@ -129,6 +135,7 @@ int main(int argc, char *argv[])
       {"debug", required_argument, 0, 'D'},
       {"force", no_argument, 0, 'f'},
       {"help", no_argument, 0, 'h'},
+      {"invert", no_argument, 0, 'i'},
       {"layout", required_argument, 0, 'l'},
       {"level", no_argument, 0, 'L'},
       {"verbose", no_argument, 0, 'v'},
@@ -136,7 +143,7 @@ int main(int argc, char *argv[])
       {0, 0, 0, 0}
       };
 
-    int c = getopt_long( argc, argv, "14ab:CD:fhLl:o:R:s:t:Vvx:",
+    int c = getopt_long( argc, argv, "14ab:CD:fhiLl:o:R:s:t:Vvx:",
                          long_options, 0 );
     if( c == -1 ) break;		// all options processed
 
@@ -152,6 +159,7 @@ int main(int argc, char *argv[])
       case 'D': debug_level = strtol( optarg, 0, 0 ); break;
       case 'f': force = true; break;
       case 'h': show_help(); return 0;
+      case 'i': invert = true; break;
       case 'L': level = true; break;
       case 'l': layout_level = strtol( optarg, 0, 0 ); break;
       case 'o':	outfile_name = optarg; break;
@@ -167,61 +175,64 @@ int main(int argc, char *argv[])
 
   if( outfile_name )
     {
-    if( append ) outfile = fopen( outfile_name, "a" );
-    else if( force ) outfile = fopen( outfile_name, "w" );
-    else if( ( outfile = fopen( outfile_name, "wx" ) ) == 0 )
+    if( append ) outfile = std::fopen( outfile_name, "a" );
+    else if( force ) outfile = std::fopen( outfile_name, "w" );
+    else if( ( outfile = std::fopen( outfile_name, "wx" ) ) == 0 )
       {
-      fprintf( stderr, "Output file %s already exists.\n", outfile_name );
+      std::fprintf( stderr, "Output file %s already exists.\n", outfile_name );
       return 1;
       }
     if( !outfile )
-      { fprintf( stderr, "Cannot open %s\n", outfile_name ); return 1; }
+      { std::fprintf( stderr, "Cannot open %s\n", outfile_name ); return 1; }
     }
 
   if( exportfile_name )
     {
-    exportfile = fopen( exportfile_name, "w" );
+    exportfile = std::fopen( exportfile_name, "w" );
     if( !exportfile )
-      { fprintf( stderr, "Cannot open %s\n", exportfile_name ); return 1; }
-    fprintf( exportfile, "# Ocr Results File. Created by GNU %s version %s\n",
-             PROGRAM_NAME, PROGRAM_VERSION );
+      { std::fprintf( stderr, "Cannot open %s\n", exportfile_name ); return 1; }
+    std::fprintf( exportfile, "# Ocr Results File. Created by GNU %s version %s\n",
+                  PROGRAM_NAME, PROGRAM_VERSION );
     }
 
 // process any remaining command line arguments (input files)
   do {
     if( optind < argc )
       {
-      if( infile != stdin ) fclose( infile );
-      if( strcmp( argv[optind], "-" ) == 0 ) infile = stdin;
-      else if( ( infile = fopen( argv[optind], "r" ) ) == 0 )
-	{ fprintf(stderr,"Cannot open %s\n",argv[optind]); return 1; }
+      if( infile != stdin ) std::fclose( infile );
+      infile_name = argv[optind];
+      if( strcmp( infile_name, "-" ) == 0 ) infile = stdin;
+      else infile = std::fopen( infile_name, "r" );
+      if( !infile )
+        { std::fprintf( stderr, "Cannot open %s\n", infile_name ); return 1; }
       ++optind;
       }
 
-// 'infile' contains the scanned text image to be converted to ascii
-// (in pbm format).
-// 'outfile' is the destination for the ascii version of the scanned
-// text image. (Or for a pbm file if debugging).
+// 'infile' contains the scanned image (in pbm format) to be converted
+// to text.
+// 'outfile' is the destination for the text version of the scanned
+// image. (or for a pbm file if debugging).
+// 'exportfile' is the Ocr Results File.
 
     try
       {
-      Bitmap page_image( infile );
+      Bitmap page_image( infile, invert );
 
       // make a list of text blocks
       page_image.analyse_layout( layout_level );
       if( verbose )
-        fprintf( stderr, "number of text blocks %d\n", page_image.rectangles() );
+        std::fprintf( stderr, "number of text blocks %d\n", page_image.rectangles() );
 
       if( rindex >= page_image.rectangles() )
         {
-        fprintf( stderr,"This page has only %d text blocks(s)\n",
-                 page_image.rectangles() );
+        std::fprintf( stderr,"This page has only %d text block(s)\n",
+                      page_image.rectangles() );
         return 1;
         }
 
       if( level )
-        fprintf( stderr, "\nslope_best = %d\n",
-                 page_image.horizontalify( verbose ) );
+        std::fprintf( stderr, "\nslope_best = %d\n",
+                      page_image.horizontalify( verbose ) );
 
       if( scale != 0 )
         {
@@ -250,8 +261,11 @@ int main(int argc, char *argv[])
         }
 
       if( exportfile )
-        fprintf( exportfile, "total blocks %d\n",
-                 ( rindex < 0 ) ? page_image.rectangles() : 1 );
+        {
+        std::fprintf( exportfile, "source file %s\n", basename( infile_name ) );
+        std::fprintf( exportfile, "total blocks %d\n",
+                      ( rindex < 0 ) ? page_image.rectangles() : 1 );
+        }
 
       // call the character recognizer for every rectangle of text
       for( int c = 0; c < page_image.rectangles(); ++c )
@@ -260,17 +274,18 @@ int main(int argc, char *argv[])
           if( exportfile )
             {
             const Rectangle & r = page_image.rectangle_vector()[c];
-            fprintf( exportfile, "block %d %d %d %d %d\n", c + 1,
-                     r.left(), r.top(), r.width(), r.height() );
+            std::fprintf( exportfile, "block %d %d %d %d %d\n", c + 1,
+                          r.left(), r.top(), r.width(), r.height() );
             }
           Blockmap blockmap( page_image, c, debug_level );
           ocr( blockmap, outfile, exportfile, debug_level );
           }
       }
 
-    catch( Bitmap::Error e ) { fputs( e.s, stderr ); return 2; }
+    catch( Bitmap::Error e ) { std::fputs( e.s, stderr ); return 2; }
     }
   while( optind < argc );
-  fclose( outfile );
+  std::fclose( outfile );
+  if( exportfile ) std::fclose( exportfile );
   return 0;
   }
