@@ -201,6 +201,33 @@ bool Profile::decreasing( int i ) throw()
   }
 
 
+bool Profile::isconcave() throw()
+  {
+  if( !valid_isconcave )
+    {
+    valid_isconcave = true; _isconcave = false; if( !valid ) initialize();
+    if( samples() < 5 ) return _isconcave;
+    int dmax = -1, l = 0, r = 0;
+
+    for( int i = pos( 10 ); i <= pos( 90 ); ++i )
+      {
+      if( data[i] > dmax ) { dmax = data[i]; l = r = i; }
+      else if( data[i] == dmax ) { r = i; }
+      }
+    if( l > r || l < pos( 25 ) || r > pos( 75 ) ) return _isconcave;
+    if( data[pos(10)] >= dmax || data[pos(90)] >= dmax ) return _isconcave;
+    int imax = ( l + r ) / 2;
+
+    for( int i = pos( 10 ); i < imax; ++i )
+      if( data[i] > data[i+1] ) return _isconcave;
+    for( int i = pos( 90 ); i > imax; --i )
+      if( data[i] > data[i-1] ) return _isconcave;
+    _isconcave = true;
+    }
+  return _isconcave;
+  }
+
+
 bool Profile::isconvex() throw()
   {
   if( !valid_isconvex )
@@ -247,7 +274,7 @@ bool Profile::isflat() throw()
     valid_isflat = true; _isflat = false; if( !valid ) initialize();
     int th = ( mean() < 1 ) ? 1 : mean();
     for( int i = 2; i < samples() - 2; ++i )
-      if( abs( data[i] - th ) > 1 ) return _isflat;
+      if( std::abs( data[i] - th ) > 1 ) return _isflat;
     _isflat = true;
     }
   return _isflat;
@@ -270,7 +297,7 @@ bool Profile::isflats() throw()
     for( int i = 1; i < begin; ++i )
       if( data[i] - mean1 > 1 ) return _isflats;
     for( int i = begin; i < end; ++i )
-      if( abs( data[i] - mean1 ) > 1 ) return _isflats;
+      if( std::abs( data[i] - mean1 ) > 1 ) return _isflats;
     for( int i = end; i < samples() - 1; ++i )
       if( data[i] - mean1 > 1 ) return _isflats;
     _isflats = true;
@@ -395,7 +422,7 @@ bool Profile::istip() throw()		// FIXME
 bool Profile::isctip( int cpos ) throw()
   {
   if( !valid ) initialize();
-  if( samples() < 5 ) return false;
+  if( samples() < 5 || cpos < 25 || cpos > 75 ) return false;
   int th = ( mean() < 2 ) ? 2 : mean();
   int imax = -1, mid = ( ( samples() - 1 ) * cpos ) / 100;
 
@@ -420,10 +447,13 @@ int Profile::imaximum() throw()
   {
   if( !valid ) initialize();
   int mbegin = 0, mend, mvalue = 0;
+
   for( int i = 0; i < samples(); ++i )
     if( data[i] > mvalue ) { mvalue = data[i]; mbegin = i; }
+
   for( mend = mbegin + 1; mend < samples(); ++mend )
     if( data[mend] < mvalue ) break;
+
   return ( mbegin + mend - 1 ) / 2;
   }
 
@@ -493,7 +523,7 @@ bool Profile::straight( int * _dy ) throw()
   for( int i = xl + 1; i < xr; ++i )
     {
     int y = yl; if( dx ) y += ( dy * ( i - xl ) / dx );
-    int d = abs( y - data[i] );
+    int d = std::abs( y - data[i] );
     if( d > 2 || ( d > 1 && --faults <= 0 ) ) return false;
     }
   *_dy = dy;
