@@ -26,8 +26,8 @@ Rectangle::Rectangle( int l, int t, int r, int b ) throw()
   {
   if( r < l || b < t )
     {
-    fprintf( stderr, "l = %d, t = %d, r = %d, b = %d\n", l, t, r, b );
-    internal_error( "bad parameter building a Rectangle" );
+    std::fprintf( stderr, "l = %d, t = %d, r = %d, b = %d\n", l, t, r, b );
+    Ocrad::internal_error( "bad parameter building a Rectangle" );
     }
   _left = l; _top = t; _right = r; _bottom = b;
   }
@@ -35,28 +35,28 @@ Rectangle::Rectangle( int l, int t, int r, int b ) throw()
 void Rectangle::left( int l ) throw()
   {
   if( l > _right )
-    internal_error( "left, bad parameter resizing a Rectangle" );
+    Ocrad::internal_error( "left, bad parameter resizing a Rectangle" );
   _left = l;
   }
 
 void Rectangle::top( int t ) throw()
   {
   if( t > _bottom )
-    internal_error( "top, bad parameter resizing a Rectangle" );
+    Ocrad::internal_error( "top, bad parameter resizing a Rectangle" );
   _top = t;
   }
 
 void Rectangle::right( int r ) throw()
   {
   if( r < _left )
-    internal_error( "right, bad parameter resizing a Rectangle" );
+    Ocrad::internal_error( "right, bad parameter resizing a Rectangle" );
   _right = r;
   }
 
 void Rectangle::bottom( int b ) throw()
   {
   if( b < _top )
-    internal_error( "bottom, bad parameter resizing a Rectangle" );
+    Ocrad::internal_error( "bottom, bad parameter resizing a Rectangle" );
   _bottom = b;
   }
 
