@@ -18,7 +18,6 @@
 
 #include <cstdio>
 #include <list>
-#include <map>
 #include <vector>
 #include "common.h"
 #include "rectangle.h"
@@ -41,14 +40,15 @@ Textblock::Textblock( const std::list< Block > & block_list ) throw()
     if( p->height() > 20 * p->width() ) continue;
     if( p->width() > 20 * p->height() ) continue;
     Character c( *p );
-    if( prev && c.right() < prev->left() )
+    if( prev && ( c.right() < prev->left() || c.top() > prev->bottom() ) )
       {
       int i;
       for( i = 0; i < line( current_line ).characters(); ++i )
         {
         Character & ci = line( current_line ).character(i);
         if( c.top() <= ci.bottom() &&
-            ( i > 0 || ci.height() <= 2 * c.height() ) ) break;
+            ( i > 0 || ci.height() <= 2 * c.height() ||
+              ci.height() <= 2 * line( current_line ).mean_height() ) ) break;
         }
       if( i >= line( current_line ).characters() ) prev = 0;
       }
@@ -64,7 +64,8 @@ Textblock::Textblock( const std::list< Block > & block_list ) throw()
     bool joined = false;
     Textline & line1 = line( current_line );
     Textline & line2 = line( current_line + 1 );
-    if( line1.mean_height1() * 2 < line2.mean_height() )
+    if( line1.characters() <= 2 * line2.characters() &&
+        2 * line1.mean_height1() < line2.mean_height() )
       for( int i1 = 0 ; !joined && i1 < line1.characters(); ++i1 )
         {
         Character & c1 = line1.character( i1 );
@@ -155,10 +156,32 @@ Textblock::Textblock( const std::list< Block > & block_list ) throw()
   }
 
 
+int Textblock::alnum_percent() const throw()
+  {
+  int total = 0;
+  for( int i = 0; i < lines(); ++i )
+    {
+    Textline & line1 = line(i);
+    for( int j = 0; j < line1.characters(); ++j )
+      if( line1.character(j).isalnum() ) ++total;
+    }
+  if( total == 0 ) return total;
+  return ( 100 * total ) / characters();
+  }
+
+
+int Textblock::characters() const throw()
+  {
+  int total = 0;
+  for( int i = 0; i < lines(); ++i ) total += line(i).characters();
+  return total;
+  }
+
+
 void Textblock::insert_line( const int i ) throw()
   {
   if( i < 0 || i > lines() )
-    internal_error( "insert_line, index out of bounds" );
+    Ocrad::internal_error( "insert_line, index out of bounds" );
   data.insert( data.begin() + i, Textline() );
   }
 
@@ -166,7 +189,7 @@ void Textblock::insert_line( const int i ) throw()
 void Textblock::delete_line( const int i ) throw()
   {
   if( i < 0 || i > lines() )
-    internal_error( "delete_line, index out of bounds" );
+    Ocrad::internal_error( "delete_line, index out of bounds" );
   data.erase( data.begin() + i );
   }
 
@@ -174,7 +197,7 @@ void Textblock::delete_line( const int i ) throw()
 Textline & Textblock::line( const int i ) const throw()
   {
   if( i < 0 || i > lines() )
-    internal_error( "line, index out of bounds" );
+    Ocrad::internal_error( "line, index out of bounds" );
   return data[i];
   }
 
@@ -182,26 +205,26 @@ Textline & Textblock::line( const int i ) const throw()
 void Textblock::print( FILE * outfile, const bool graph, const bool recursive )
 								const throw()
   {
-  if( graph || recursive ) fprintf( outfile, "%d lines\n\n", lines() );
+  if( graph || recursive ) std::fprintf( outfile, "%d lines\n\n", lines() );
 
   for( int i = 0; i < lines(); ++i )
     {
     if( graph || recursive )
-      fprintf( outfile, "%d characters in line %d\n", line(i).characters(), i+1 );
+      std::fprintf( outfile, "%d characters in line %d\n", line(i).characters(), i+1 );
     line(i).print( outfile, graph, recursive );
     }
-  fputs( "\n", outfile );
+  std::fputs( "\n", outfile );
   }
 
 
 void Textblock::xprint( FILE * exportfile ) const throw()
   {
-  fprintf( exportfile, "lines %d\n", lines() );
+  std::fprintf( exportfile, "lines %d\n", lines() );
 
   for( int i = 0; i < lines(); ++i )
     {
-    fprintf( exportfile, "line %d chars %d height %d\n", i + 1,
-             line(i).characters(), line(i).mean_height() );
+    std::fprintf( exportfile, "line %d chars %d height %d\n", i + 1,
+                  line(i).characters(), line(i).mean_height() );
     line(i).xprint( exportfile );
     }
   }
